% Perform source-filter separation on an entire sentence
clc
clear
close all

[x,fs] = wavread('rms_arctic_a0013.wav');

x = x';

% Preemphasis filter
% x = filter([1 -0.9],1,x);

wlen  = round(30e-3*fs);
wshft = round(1e-3*fs);
win   = hamming(wlen)';
nfft  = 2^10;
nlpc  = 12;

%% Generate LPC spectral envelope for each frame

[stft_x]        = stft(x,win,wshft,nfft);
[a_x lpc_x e_x] = lpc_mat(x,win,wshft,nlpc,nfft,fs);

figure
imagesc(flipud(log(abs(stft_x(:,1:nfft/2)))')); colorbar
figure
imagesc(flipud(log(abs(lpc_x(:,1:nfft/2)))')); colorbar


%%  Inverse filtering and playing the residual
nlpc  = 20;
wlen  = round(30e-3*fs);
wshft = round(1e-3*fs);

[a, r] = lpc_if(x, nlpc, wlen, wshft);
lpc_x = [];
for f=1:size(a,1)
  lpc_x(f,:) = freqz(1,a(f,:),nfft/2,fs)';
end

subplot(411); plot(x/std(x)); hold on; plot(r/std(r),'r'); axis tight
subplot(412); spectrogram(x,wlen,wlen-wshft,nfft,fs,'yaxis'); colorbar
subplot(413); spectrogram(r,wlen,wlen-wshft,nfft,fs,'yaxis'); colorbar
subplot(414); imagesc(flipud(log(abs(lpc_x(:,1:nfft/2)))')); colorbar

soundsc(r,fs)

%% Resynhtesize the signal with the true residual and white noise
nlpc  = 20;
wlen  = round(30e-3*fs);
wshft = round(1e-3*fs);

[a, r] = lpc_if(x, nlpc, wlen, wshft);

[xhat1] = lpc_resynth(r,              a, wlen, wshft);
[xhat2] = lpc_resynth(randn(size(r)), a, wlen, wshft);

subplot(411); plot(x); hold on; plot(xhat1, 'g.'); legend('signal', 'reconstruction'); axis tight
subplot(412); spectrogram(x,    wlen,wlen-wshft,nfft,fs,'yaxis'); 
subplot(413); spectrogram(xhat1,wlen,wlen-wshft,nfft,fs,'yaxis'); 
subplot(414); spectrogram(xhat2,wlen,wlen-wshft,nfft,fs,'yaxis'); 

soundsc(x,fs)
soundsc(xhat1,fs)
soundsc(xhat2,fs)
